@@ -1,8 +1,16 @@
 from functools import partial, wraps
 from collections import deque
-from typing import Union, Callable, Iterable
+from typing import Union, Callable, Iterable, Any
+from dataclasses import dataclass
+from lined.base import Fnode
+from lined.util import func_name, partial_plus
 
-from lined.util import func_name
+
+def add_name(obj, name=None):
+    if name is None:
+        name = type(obj).__name__
+    obj.__name__ = name
+    return obj
 
 
 def keys_extractor(keys):
@@ -53,6 +61,15 @@ def iterate(iterable: Iterable):
         pass
 
 
+def side_call(x, callback):
+    callback(x)
+    return x
+
+
+print_and_pass_on = partial_plus(side_call, callback=print, __name__='print_and_pass_on',
+                                 __doc__="Passes input through to output, but prints before outputing")
+
+
 # Function transformers ###################################################################
 
 
@@ -69,6 +86,32 @@ def mywraps(func, name=None, doc_prefix=""):
     return wrapper
 
 
+def tail_io(func):
+    """Will apply function only to the tail of tuple inputs, still passing the header on.
+    That is, from a ``x -> func(x)`` function, you get a ``(*header, x) -> (*header, func(x))`` function.
+
+    >>> def foo(x):
+    ...    return x * 2
+    >>>
+    >>> foo('boo')
+    'booboo'
+    >>> new_foo = tail_io(foo)
+    >>> new_foo((7, 'boo'))
+    (7, 'booboo')
+    >>> new_foo(('all', 'items', 'but', 'the', 'last', 'are', 'just', 'passed', 'on', 'boo'))
+    ('all', 'items', 'but', 'the', 'last', 'are', 'just', 'passed', 'on', 'booboo')
+
+    """
+
+    @mywraps(func)
+    def _func(input):
+        *header, real_input = input
+        out = func(real_input)
+        return *header, out
+
+    return _func
+
+
 def iterize(func, name=None):
     """From an Input->Ouput function, makes a Iterator[Input]->Itertor[Output]
     Some call this "vectorization", but it's not really a vector, but an iterable, thus the name.
@@ -126,13 +169,20 @@ def iterize(func, name=None):
 generator_version = iterize  # back compatibility alias
 
 
+def mk_filter(filter_func=None):
+    return partial_plus(filter, filter_func, __name__='mk_filter', __doc__='Makes a filter with a fixed filt func.')
+
+
 def map_star(func):
     """Make a func(args) function out of a func(*args) one.
     Also known as singularize_arg_input.
 
     >>> def foo(a, b):
     ...     return a + b
-    >>> 
+    >>> singularized_foo = map_star(foo)
+    >>> singularized_foo((2, 3))
+    5
+    >>> assert singularized_foo([2, 3]) == singularized_foo({2, 3}) == foo(2, 3)
     """
 
     @mywraps(func, doc_prefix=f"singularize_arg_input version of {func_name(func)}")
@@ -145,12 +195,83 @@ def map_star(func):
 singularize_arg_input = map_star  # alias
 
 
+class Enumerate:
+    """Decorator a function so it enumerates the number of calls.
+    Or in general, returns (cursor, func(x)) instead of just func(x), where the start and step of the cursor can
+    be defined (default is start=0 and step=1)
+
+    >>> def foo(x):
+    ...    return x * 2
+    >>> new_foo = Enumerate(foo)
+    >>> new_foo('ha')
+    (0, 'haha')
+    >>> new_foo('ho')
+    (1, 'hoho')
+    >>> enum_foo_with_step = Enumerate(foo, start=3, step=7)
+    >>> enum_foo_with_step('z')
+    (3, 'zz')
+    >>> enum_foo_with_step(11)
+    (10, 22)
+    """
+
+    def __init__(self, func, start=0, step=1):
+        self.func = func
+        self.cursor = start
+        self.step = step
+
+    def __call__(self, *args, **kwargs):
+        current_cursor = self.cursor
+        out = self.func(*args, **kwargs)
+        self.cursor += self.step
+        return current_cursor, out
+
+
+def with_cursor(func, start=0, step=1):
+    """Decorator a function so it enumerates the number of calls.
+    Or in general, returns (cursor, func(x)) instead of just func(x), where the start and step of the cursor can
+    be defined (default is start=0 and step=1)
+
+    >>> def foo(x):
+    ...    return x * 2
+    >>> new_foo = with_cursor(foo)
+    >>> new_foo('ha')
+    (0, 'haha')
+    >>> new_foo('ho')
+    (1, 'hoho')
+    >>> enum_foo_with_step = with_cursor(foo, start=3, step=7)
+    >>> enum_foo_with_step('z')
+    (3, 'zz')
+    >>> enum_foo_with_step(11)
+    (10, 22)
+    """
+
+    @wraps(func)
+    def _func(*args, **kwargs):
+        current_cursor = _func.cursor
+        out = func(*args, **kwargs)
+        _func.cursor += step
+        return current_cursor, out
+
+    _func.cursor = start
+    return _func
+
+
+Stats = Any
+from typing import Optional
+
+from typing import cast
+
+_no_value_specified_sentinel = cast(int, object())
+
+
+# _no_value_specified_sentinel = object()
+
 class BufferStats(deque):
     """A callable (fifo) buffer. Calls add input to it, but also returns a function of it's contents.
 
     What "add" means is configurable (through ``add_new_val`` arg). Default is append, but can be extend etc.
 
-    >>> bs = BufferStats(4, sum)
+    >>> bs = BufferStats(maxlen=4, func=sum)
     >>> list(map(bs, range(7)))
     [0, 1, 3, 6, 10, 14, 18]
 
@@ -161,16 +282,16 @@ class BufferStats(deque):
 
     More examples:
 
-    >>> list(map(BufferStats(4, ''.join), 'abcdefgh'))
+    >>> list(map(BufferStats(maxlen=4, func=''.join), 'abcdefgh'))
     ['a', 'ab', 'abc', 'abcd', 'bcde', 'cdef', 'defg', 'efgh']
 
     >>> from math import prod
-    >>> list(map(BufferStats(4, prod), range(7)))
+    >>> list(map(BufferStats(maxlen=4, func=prod), range(7)))
     [0, 0, 0, 0, 24, 120, 360]
 
     With a different ``add_new_val`` choice.
 
-    >>> bs = BufferStats(4, ''.join, add_new_val=deque.appendleft)
+    >>> bs = BufferStats(maxlen=4, func=''.join, add_new_val=deque.appendleft)
     >>> list(map(bs, 'abcdefgh'))
     ['a', 'ba', 'cba', 'dcba', 'edcb', 'fedc', 'gfed', 'hgfe']
 
@@ -178,7 +299,7 @@ class BufferStats(deque):
     In the following, also see how we use iterize to get a function that takes an iterator and returns an iterator
 
     >>> from lined import iterize
-    >>> window_stats = iterize(BufferStats(4, ''.join, add_new_val=deque.extend))
+    >>> window_stats = iterize(BufferStats(maxlen=4, func=''.join, add_new_val=deque.extend))
     >>> chks = ['a', 'bc', 'def', 'gh']
     >>> for x in window_stats(chks):
     ...     print(x)
@@ -193,7 +314,13 @@ class BufferStats(deque):
 
     """
 
-    def __init__(self, maxlen, func: Callable = sum, add_new_val: Callable = deque.append):
+    # __name__ = 'BufferStats'
+
+    def __init__(self,
+                 values=(),
+                 maxlen: int = _no_value_specified_sentinel,
+                 func: Callable = sum,
+                 add_new_val: Callable = deque.append):
         """
 
         :param maxlen: Size of the buffer
@@ -202,12 +329,44 @@ class BufferStats(deque):
             Is usually a deque method (``deque.append`` by default, but could be ``deque.extend``,
             ``deque.appendleft`` etc.). Can also be any other function that has a valid (self, new_val) signature.
         """
-        super().__init__(maxlen=maxlen)
+        if maxlen is _no_value_specified_sentinel:
+            raise TypeError("You are required to specify maxlen")
+        if not isinstance(maxlen, int):
+            raise TypeError(f"maxlen must be an integer, was: {maxlen}")
+
+        super().__init__(values, maxlen=maxlen)
         self.func = func
         if isinstance(add_new_val, str):
             add_new_val = getattr(self, add_new_val)  # add_new_val is a method of deque
         self.add_new_val = add_new_val
+        self.__name__ = 'BufferStats'
 
-    def __call__(self, new_val):
+    def __call__(self, new_val) -> Stats:
         self.add_new_val(self, new_val)  # add the new value
         return self.func(self)
+
+
+def is_not_none(x):
+    return x is not None
+
+
+def return_buffer_on_stats_condition(stats: Stats,
+                                     buffer: Iterable,
+                                     cond: Callable = is_not_none,
+                                     else_val=None):
+    if cond(stats):
+        return buffer
+    else:
+        return else_val
+
+
+# @add_name
+@dataclass
+class Segmenter:
+    buffer: BufferStats
+    stats_buffer_callback: Callable[[Stats, Iterable], Any] = return_buffer_on_stats_condition
+    __name__ = 'Segmenter'
+
+    def __call__(self, new_val):
+        stats = self.buffer(new_val)
+        return self.stats_buffer_callback(stats, list(self.buffer))
